@@ -175,14 +175,27 @@ __non_packages__       -- list of subdirectories of the setup directory
 
 __ext_modules__        -- list of extension module objects; this should
                           not be needed, use __ext_names__
-__ext_names__          -- list of path names to extension modules
-                          (starting from the setup.py directory); the
-                          source code for each module is found by using
-                          the __ext_src__ variable
-__ext_src__            -- pathname to extension source files, usually
-                          relative (meaning relative to the extension
-                          directory); defaults to 'src', meaning the
-                          'src' subdirectory of the extension directory
+__ext_names__          -- list of names of extension modules; dotted names
+                          are interpreted as modules within packages (so
+                          ``foo`` is a top-level module, but ``pkg.foo`` is
+                          a module in package ``pkg``); the source code for
+                          each module is found by using the __ext_src__
+                          variable to name a subdirectory of the extension
+                          module directory (which will be the corresponding
+                          source root or package directory).
+__ext_src__            -- pathname to extension source files, relative to
+                          the source root; if omitted, extension source files
+                          will be looked for in the 'src' subdirectory of each
+                          extension module directory (this should be the usual
+                          case, although it is somewhat different that the
+                          assumed default in the Python documentation; it
+                          seems neater to keep each extension module's source
+                          code near the module's own location in the build
+                          tree, and difficulties that might otherwise arise
+                          with doing this are avoided by SetupHelper's
+                          automatically excluding extension source directories
+                          from the list of packages, as noted under the
+                          ``__package_data__`` variable above).
 
 __scripts__            -- list of scripts to be included; this should
                           generally not be used, use __script_root__
@@ -250,7 +263,7 @@ import sys
 import os
 from itertools import imap, izip
 
-SETUPHELPER_VERSION = (0, 4, 4)
+SETUPHELPER_VERSION = (0, 5, 1)
 
 def version_tuple(ver_s, dev=False):
     """
@@ -452,6 +465,20 @@ archive_extensions = {
 
 install_scripts_dir = None # see ugly hack below
 
+# Mixin class to pretty-print Extension instances
+
+class ExtensionMixin: # not a new-style class since Extension isn't
+    """
+    Mixin class adds ``__repr__`` and ``__str__`` methods to Extension
+    so it will print human-readable output if the ``--verbose`` option
+    is used. Note that not all of the instance variables are output,
+    just the minimum to see if the instance makes sense.
+    """
+    
+    def __repr__(self):
+        return 'Extension(%r, sources=%s)' % (self.name, self.sources)
+    __str__ = __repr__
+
 def distutils_default():
     """ Support function for Python-provided distutils. """
     
@@ -822,12 +849,15 @@ def distutils_default():
                     print >>sys.stderr, \
                         "Could not find requirement %s; program may not run." % req
     
+    class SHExtension(ExtensionMixin, Extension):
+        pass
+    
     def setup_wrapper(**kwargs):
         """ Wrapper for distutils setup function. """
         if 'name' in kwargs:
             setup(**kwargs)
     
-    return (RequiresHelper, Extension, setup_wrapper)
+    return (RequiresHelper, SHExtension, setup_wrapper)
 
 def distutils_setuptools():
     """ Support function for setuptools. """
@@ -843,6 +873,9 @@ def distutils_setuptools():
     from setuptools import setup
     from setuptools.extension import Extension
     
+    class SHExtension(ExtensionMixin, Extension):
+        pass
+    
     def setup_wrapper(**kwargs):
         """ Wrapper for setuptools setup function. """
         if 'name' in kwargs:
@@ -854,7 +887,7 @@ def distutils_setuptools():
             for req in kwargs['install_requires']:
                 main([req] + opts)
     
-    return (None, Extension, setup_wrapper)
+    return (None, SHExtension, setup_wrapper)
 
 # This will be used to process the __license__ variable below
 
@@ -904,9 +937,34 @@ class SetupHelper(object):
         'requires_class',
         'setup_func' ]
     
+    dirsep = '/'
+    pkgsep = '.'
+    
+    verbose = False
+    
+    def dir_to_os(self, dirname):
+        """ Converts distutils dirname to os dirname. """
+        return os.path.join(*dirname.split(self.dirsep))
+    
     def dir_to_pkg(self, dirname):
         """ Converts relative directory path into dotted package name. """
-        return dirname.replace('/', '.')
+        return dirname.replace(self.dirsep, self.pkgsep)
+    
+    def os_to_dir(self, dirname):
+        """ Converts os directory path into distutils dirname. """
+        return dirname.replace(os.sep, self.dirsep)
+    
+    def os_to_pkg(self, dirname):
+        """ Converts os directory path into dotted package name. """
+        return dirname.replace(os.sep, self.pkgsep)
+    
+    def pkg_to_dir(self, pkgname):
+        """ Converts dotted package name into relative directory path. """
+        return pkgname.replace(self.pkgsep, self.dirsep)
+    
+    def pkg_to_os(self, pkgname):
+        """ Converts dotted package name to os dirname. """
+        return os.path.join(*pkgname.split(self.pkgsep))
     
     def search_classifiers(self, s):
         """
@@ -1094,26 +1152,27 @@ class SetupHelper(object):
         
         # Figure out the extension modules list (do this first so we can filter
         # out extension directories from the package data list below)
-        self.__ext_modules__ = []
+        if not hasattr(self, '__ext_modules__'):
+            self.__ext_modules__ = []
         self.__ext_sourcedirs__ = []
         if hasattr(self, '__ext_names__'):
-            if not hasattr(self, '__ext_src__'):
-                self.__ext_src__ = 'src'
-            if os.path.isabs(self.__ext_src__):
-                ext_src_func = lambda extdir: self.__ext_src__
+            if hasattr(self, '__ext_src__'):
+                ext_src_func = lambda extname: self.__ext_src__
             else:
-                ext_src_func = lambda extdir: os.path.normpath(
-                    os.path.join(extdir, self.__ext_src__))
-            for extpath in self.__ext_names__:
-                extdir, extname = os.path.split(extpath)
-                srcdir = ext_src_func(extdir)
+                ext_src_func = lambda extname: os.path.normpath(
+                    os.path.join(os.path.split(self.pkg_to_os(extname))[0], 'src'))
+            for extname in self.__ext_names__:
+                srcdir = ext_src_func(extname)
                 if os.path.isdir(srcdir):
-                    self.__ext_modules__.append(self.__ext_class__(extpath,
-                        sources=[os.path.join(srcdir, filename)
+                    self.__ext_modules__.append(self.__ext_class__(extname,
+                        sources=[self.os_to_dir(os.path.join(srcdir, filename))
                             for filename in os.listdir(srcdir) if filename.endswith(".c")]))
                     self.__ext_sourcedirs__.append(srcdir)
-            del self.__ext_src__
+            if hasattr(self, '__ext_src__'):
+                del self.__ext_src__
             del self.__ext_names__
+        # TODO: if __ext_modules__ exists but no __ext_names__, parse it to
+        # populate __ext_sourcedirs__
         if not self.__ext_modules__:
             del self.__ext_modules__
         del self.__ext_class__
@@ -1145,15 +1204,16 @@ class SetupHelper(object):
             for pkgdir in self.__package_dirs__:
                 for directory, subdirs, files in os.walk(pkgdir):
                     if process_packages and ("__init__.py" in files):
-                        self.__packages__.append(self.dir_to_pkg(directory))
+                        self.__packages__.append(self.os_to_pkg(directory))
                     elif process_packagedata and ("." not in directory) \
                         and (directory not in self.__ext_sourcedirs__):
                             parentdir, thisdir = os.path.split(directory)
-                            pkg = self.dir_to_pkg(parentdir)
+                            pkg = self.os_to_pkg(parentdir)
+                            pkgdir = self.os_to_dir(directory)
                             if not pkg in self.__package_data__:
                                 self.__package_data__[pkg] = []
-                            self.__package_data__[pkg].append("%s/*.*" % thisdir)
-                            self.__manifest_in__.append("recursive-include %s *.*\n" % directory)
+                            self.__package_data__[pkg].append("%s%s*.*" % (thisdir, self.dirsep))
+                            self.__manifest_in__.append("recursive-include %s *.*\n" % pkgdir)
         if not self.__packages__:
             del self.__packages__
         if not self.__package_data__:
@@ -1172,9 +1232,10 @@ class SetupHelper(object):
             if os.path.isdir(self.__script_root__):
                 for directory, subdirs, files in os.walk(self.__script_root__):
                     if "." not in directory:
+                        scriptdir = self.os_to_dir(directory)
                         for filename in files:
                             if not filename.startswith("."):
-                                self.__scripts__.append("%s/%s" % (directory, filename))
+                                self.__scripts__.append("%s%s%s" % (scriptdir, self.dirsep, filename))
         if hasattr(self, '__script_root__'):
             del self.__script_root__
         if not hasattr(self, '__data_files__'):
@@ -1188,11 +1249,12 @@ class SetupHelper(object):
                     if datadir == self.__progname__:
                         pathprefix = "lib"
                     else:
-                        pathprefix = "share/%s" % self.__progname__
+                        pathprefix = "share%s%s" % (self.dirsep, self.__progname__)
                     for directory, subdirs, files in os.walk(datadir):
                         if files and ("." not in directory):
-                            self.__data_files__.append(("%s/%s" % (pathprefix, directory),
-                                ["%s/%s" % (directory, filename) for filename in files
+                            distdir = self.os_to_dir(directory)
+                            self.__data_files__.append(("%s%s%s" % (pathprefix, self.dirsep, distdir),
+                                ["%s%s%s" % (distdir, self.dirsep, filename) for filename in files
                                     if not filename.startswith(".")]))
                     self.__manifest_in__.append("recursive-include %s *.*\n" % datadir)
         if not self.__scripts__:
@@ -1321,6 +1383,9 @@ class SetupHelper(object):
         function call.
         """
         
+        if self.verbose:
+            for key, value in self.__setup_args__.iteritems():
+                print key, '=', value
         self.__setup_func__(**self.__setup_args__)
     
     def do_post_install(self):
@@ -1340,6 +1405,10 @@ class SetupHelper(object):
 
 def setup_main(vars, **kwargs):
     helper = SetupHelper(vars, **kwargs)
+    for opt in ('-v', '--verbose'):
+        if opt in sys.argv:
+            sys.argv.remove(opt)
+            helper.verbose = True
     helper.do_setup()
     if 'install' in sys.argv:
         helper.do_post_install()
