@@ -1,9 +1,8 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """
 Module KDEAPP -- Python KDE Application Objects
 Sub-Package GUI.TOOLKITS.KDE of Package PLIB -- Python GUI Toolkits
-Copyright (C) 2008-2011 by Peter A. Donis
+Copyright (C) 2008-2012 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -69,6 +68,29 @@ class _PKDEBaseMixin(kdeui.KMainWindow, _PKDECommunicator):
     def set_caption(self, caption):
         self.setPlainCaption(caption)
     
+    def _get_desktop_rect(self, primary=True):
+        # Correctly handle virtual desktop across multiple screens
+        desktop = self.app.desktop()
+        l = desktop.x()
+        t = desktop.y()
+        w = desktop.width()
+        h = desktop.height()
+        if desktop.isVirtualDesktop() and primary:
+            # Return the rect of the primary screen only
+            i = desktop.primaryScreen()
+            n = desktop.numScreens()
+            w = w / n
+            # NOTE: We have to check for i > 0 here because in some
+            # cases (e.g., when running in a VirtualBox), Qt thinks
+            # the desktop is "virtual" but there's only one screen and
+            # desktop.primaryScreen returns 0 instead of 1.
+            if i > 0:
+                l += w * (i - 1)
+        else:
+            i = 0
+            n = 1
+        return i, n, l, t, w, h
+    
     def sizetoscreen(self, maximized):
         if maximized:
             if self.shown:
@@ -76,19 +98,20 @@ class _PKDEBaseMixin(kdeui.KMainWindow, _PKDECommunicator):
             else:
                 self._showMax = True
         else:
-            desktop = self.app.desktop()
+            i, n, l, t, w, h = self._get_desktop_rect()
             self.resize(
-                desktop.width() - self.sizeoffset,
-                desktop.height() - self.sizeoffset)
+                w - self.sizeoffset,
+                h - self.sizeoffset)
+            self.move(l, t)
     
     def sizetoclient(self, clientwidth, clientheight):
         self.resize(clientwidth, clientheight)
     
     def center(self):
-        desktop = self.app.desktop()
+        i, n, l, t, w, h = self._get_desktop_rect()
         g = self.frameGeometry()
         x, y = g.width(), g.height()
-        self.move((desktop.width() - x)/2, (desktop.height() - y)/2)
+        self.move(l + (w - x)/2, t + (h - y)/2)
     
     def show_init(self):
         if hasattr(self, '_showMax'):
