@@ -1,9 +1,8 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """
 Module AttrDict
 Sub-Package STDLIB of Package PLIB
-Copyright (C) 2008-2011 by Peter A. Donis
+Copyright (C) 2008-2012 by Peter A. Donis
 
 Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
@@ -20,7 +19,7 @@ class AttrDict(AttrDelegate, basekeyed):
     Only attributes defined in the attribute list passed to this
     class will appear as allowed keys in the mapping. The
     mapping is immutable (since it is only supposed to be
-    "assigned" to during option parsing).
+    "assigned" to during initialization).
     """
     
     def __init__(self, keylist, obj):
